﻿Imports System
Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput

Imports Autodesk.AutoCAD.Colors

<Assembly: CommandClass(GetType(dotNetTraining_Webcast_Demos.Webcast_5_Demos))> 

Namespace dotNetTraining_Webcast_Demos
    Public Class Webcast_5_Demos

        <CommandMethod("demoAddExtensionDictionary")> _
        Public Sub addExtensionDictionaryDemo()

            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            Dim trans As Transaction = ed.Document.Database.TransactionManager.StartTransaction
            ' get an entity to add extension dictionary
            Dim getEntityResult As PromptEntityResult =
                ed.GetEntity("Select an entity to add extension dictioanry to ")

            Try
                ' check to see that everything is ok
                If (getEntityResult.Status = PromptStatus.OK) Then
                    ' get the entity
                    Dim ent As Entity = trans.GetObject(getEntityResult.ObjectId, OpenMode.ForRead)
                    ' see the if the dictionary is already there
                    If (ent.ExtensionDictionary.IsNull) Then
                        ' upgrade the entity 
                        ent.UpgradeOpen()
                        ' make the dictionary
                        ent.CreateExtensionDictionary()
                        ' getting the dictionary
                        Dim extDictionary As DBDictionary = _
                            trans.GetObject(ent.ExtensionDictionary, OpenMode.ForWrite)
                        ' create an Xrecrod
                        Dim Xrec As New Xrecord
                        ' create a res buf
                        Dim dataResBuf As ResultBuffer = _
                            New ResultBuffer(New TypedValue(DxfCode.Text, "Some Custom Data"), _
                                                        New TypedValue(DxfCode.Real, 35.4))
                        ' set the data of xrecord to the resultbuffer
                        Xrec.Data = dataResBuf
                        ' add the xrecord to the dictionary
                        extDictionary.SetAt("CustomData", Xrec)
                        ' let the transaction know about the xrecord
                        trans.AddNewlyCreatedDBObject(Xrec, True)

                    End If

                End If

                trans.Commit()

            Catch ex As System.Exception

                ed.WriteMessage(ex.ToString)

            Finally

                trans.Dispose()

            End Try

        End Sub

        <CommandMethod("demoGetExtensionDictionary")> _
        Public Sub getExtensionDictionaryDemo()

            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            Dim trans As Transaction = ed.Document.Database.TransactionManager.StartTransaction

            Dim getEntityResult As PromptEntityResult = ed.GetEntity("Select an entity ")

            Try

                If (getEntityResult.Status = PromptStatus.OK) Then

                    Dim ent As Entity = trans.GetObject(getEntityResult.ObjectId, OpenMode.ForRead)
                    ' make sure the extension dictionary is there
                    If (Not ent.ExtensionDictionary.IsNull) Then
                        ' upgrade 
                        ent.UpgradeOpen()
                        ' getting the extension dictionary
                        Dim extDictionary As DBDictionary = _
                            trans.GetObject(ent.ExtensionDictionary, OpenMode.ForRead)
                        ' get object id
                        Dim xRec_ObjId = extDictionary.GetAt("CustomData")
                        ' instantiating the xrecord
                        Dim Xrec As Xrecord = trans.GetObject(xRec_ObjId, OpenMode.ForRead)
                        ' get the res buf
                        Dim dataResBuf As ResultBuffer = Xrec.Data
                        ' get the enumerator
                        Dim resBufEnum As ResultBufferEnumerator = dataResBuf.GetEnumerator()
                        ' get values in the resbuf
                        Do While resBufEnum.MoveNext

                            Dim typVal As TypedValue

                            typVal = CType(resBufEnum.Current, TypedValue)

                            ed.WriteMessage(vbCrLf & "Value in Xrecord is: " & typVal.Value.ToString & vbCrLf)
                        Loop
                    Else

                        ed.WriteMessage("Selected entity does not have an extension dictionary " & vbCrLf)

                    End If

                End If

                trans.Commit()

            Catch ex As System.Exception

                ed.WriteMessage(ex.ToString)

            Finally

                trans.Dispose()

            End Try

        End Sub

        <CommandMethod("demoAddDictionaryToNOD")> _
        Public Sub addExtensionDictionaryToNODDemo()
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            Dim trans As Transaction = ed.Document.Database.TransactionManager.StartTransaction

            Try

                Dim namedObjDictionary As DBDictionary = _
                   trans.GetObject(ed.Document.Database.NamedObjectsDictionaryId, OpenMode.ForRead)

                If (Not namedObjDictionary.Contains("CustomDictionary")) Then

                    namedObjDictionary.UpgradeOpen()
                    ' create a new dictionary
                    Dim customDictionary As DBDictionary = New DBDictionary

                    Dim Xrec As New Xrecord
                    ' create res buf
                    Dim dataResBuf As ResultBuffer = _
                          New ResultBuffer(New TypedValue(DxfCode.Text, "Some Custom Data"), _
                                            New TypedValue(DxfCode.Real, 35.4))
                    ' add the res buf to xrecord
                    Xrec.Data = dataResBuf


                    namedObjDictionary.SetAt("CustomDictionary", customDictionary)

                    trans.AddNewlyCreatedDBObject(customDictionary, True)

                    customDictionary.SetAt("CustomXrecord", Xrec)

                    trans.AddNewlyCreatedDBObject(Xrec, True)


                Else

                    ed.WriteMessage("CustomDictionary (DBDictionary) alread in Named Objects Dictionary " & vbCrLf)

                End If

                trans.Commit()

            Catch ex As System.Exception
                ed.WriteMessage(ex.ToString)

            Finally
                trans.Dispose()

            End Try

        End Sub

        <CommandMethod("demoGetType")> _
        Public Sub getTypeDemo()
            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            Dim trans As Transaction = ed.Document.Database.TransactionManager.StartTransaction

            Dim getEntityResult As PromptEntityResult = ed.GetEntity("Select an entity : ")

            Try
                If (getEntityResult.Status = PromptStatus.OK) Then

                    Dim ent As Entity = trans.GetObject(getEntityResult.ObjectId, OpenMode.ForRead)

                    ed.WriteMessage("The type of entity you selected is " & ent.GetType().Name)

                End If

                trans.Commit()

            Catch ex As System.Exception

                ed.WriteMessage(ex.ToString)

            Finally

                trans.Dispose()

            End Try
        End Sub



        <CommandMethod("demoIterateMSpace")> _
        Public Sub iterateMSpaceDemo()

            Dim ed As Editor = Application.DocumentManager.MdiActiveDocument.Editor

            Dim db As Database = ed.Document.Database


            Dim trans As Transaction = db.TransactionManager.StartTransaction()

            Try
                ' get the block table
                Dim blkTable As BlockTable = trans.GetObject(db.BlockTableId, OpenMode.ForRead)
                ' get model space
                Dim btrModelSpace As BlockTableRecord = _
                    trans.GetObject(blkTable(BlockTableRecord.ModelSpace), OpenMode.ForRead)
                ' create an object id
                Dim objId As ObjectId = ObjectId.Null

                ' iterate through model space
                For Each objId In btrModelSpace
                    ' make a DBObject
                    Dim dbObj As DBObject = trans.GetObject(objId, OpenMode.ForRead)
                    ' test to see if it is a line
                    If dbObj.GetType.Name = "Line" Then
                        ' create line Line
                        Dim lineObj As Line = CType(dbObj, Line)
                        ' open for write
                        lineObj.UpgradeOpen()
                        ' change the color
                        lineObj.Color = Color.FromColorIndex(ColorMethod.ByAci, 1)

                    End If

                Next


                trans.Commit()

            Catch ex As System.Exception

                ed.WriteMessage(ex.ToString())

            Finally

                trans.Dispose()

            End Try

        End Sub

    End Class
End Namespace
